% Rocio Alvarez, Maximo Camacho, Manuel Ruiz 
% www.um.es/econometria/Maximo
% October, 18th 2016 @

%  This MATLAB file reads in data, sets options, and calls numerical
%  optimization routine for numerical estimation of the univariate
%  switching model:
% 
%         Yt = m(st) + a1*[Yt-1 - m(st-1)] +a2*[Yt-2 - m(st-2)] + Et   
%         Et ~ N(0,V(st))   st=1,2,3      

clear;clc;
je=1;
global ns; global pphi; global captst;global y;
graphs=1;  % graphs=0 to skip graphs
outp=1;    % outp=0 to skip outputs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       LOADING DATA                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
level=0.05; % Confidence level used in confidence intervals
filename='data_3s_all.xlsx';  % date qtr, T-bill (ann), CPI (1947.1-1990.4; 176 obs)
sheet=1;
xlRange='A2:C332';
data=xlsread(filename,sheet,xlRange);

fecha=data(:,1);
ex_r=data(:,2);   
inf=data(:,3);
y=ex_r-inf;      % Ex-post interest rates
capt=length(y);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                OPTIMIZATION PROCEDURES                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pphi=2;        % pphi is the number of lags in autoregression
ns=3;          % ns is the number of primitive states
n=ns^(pphi+1); % n is the dimension of the state vector 
nk=pphi+1;     % nk is the first obs for which the like is evaluated 
captst=capt-pphi;    % captst is the effective sample size
mu0=-1.5;mu1=1.3;mu2=4;  % Initial values from Kim-Nelson's book
phi1=0.1;phi2=0.1;
sig0=2;sig1=1;sig2=2;
% p00=3;p01=-12;p10=9;p11=14;p20=-22;p21=-4;
 p00=2;p01=1;p10=1;p11=2;p20=-4;p21=-4;
 
startval=[mu0 mu1 mu2 phi1 phi2 sig0 sig1 sig2 p00 p01 p10 p11 p20 p21];

options=optimset('Display','off','MaxIter',500,'TolX',10^-6,'TolFun',10^-6);
[theta,fval,exitflag,output,grad,hessian]=fminunc(@ofn,startval,options,y);
theta=theta';
thn=trans2(theta);

[skif2,hp]=ofnprec(theta',y);
hpp=hp';
skif=skif2*hpp(:,1:ns);
[yv,re,ye]=makeresi(y,thn'); % Ex-ante interest rates

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                         VARCOV                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if je>0;   
   if min(eig(hessian))<=0;
      clc;
      disp('Negative Hessian is not positive definite');
      break;
   end; 
   hi=inv(hessian);
   std_theta=diag(hi).^0.5;     
   gr=Grad(@trans2,theta);
   Hfin=gr*hi*gr';
   std_thn=real(diag(Hfin).^0.5);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 Smoothed probs                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
skis=smoother_Hamilton(skif,thn);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      Distribution filtered and smoothed probs            %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[Gt,vari]=delta_filtered(y,thn,Hfin);
[Sm,varism]=delta_smoothed(y,thn,Hfin);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           CI                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
indi=1;     % indi=1 for normal and 0 for beta distribuition
ci=CI_calcula(Gt,vari,level,indi);
ci2=CI_calcula(Sm,varism,level,indi);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           Graphs                          %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;
if graphs>0

j=1;
figure;plot(fecha(nk:capt),[skif(:,j)]);
title('filtered probabilities');
figure;plot(fecha(nk:capt),vari(:,j));
title('variance of filtered probabilities');
figure;plot(fecha(nk:capt),ci(:,(j-1)*2+1:j*2));
title('Confidence intervals filtered');

figure;plot(fecha(nk:capt),[skis(:,j)]);
title('smoothed probabilities');
figure;plot(fecha(nk:capt),varism(:,j));
title('variance of smoothed probabilities');
figure;plot(fecha(nk:capt),ci2(:,(j-1)*2+1:j*2));
title('Confidence intervals smoothed');

[yv,re,ye]=makeresi(y,thn');
figure;plot(fecha(nk:capt),[ye yv]);
title('Ex-ante and Ex-post interest rate');

end;

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %                         Outputs                          %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if outp>0
    captst=length(y(nk:end));
    filename='results_int_rates.xlsx';
    A={'Date','Filtered','var(filtered)','ci_l','ci_u','smoothed','var(smoothed)','ci_l','ci_u'};
    B=[fecha(nk:capt) skif(:,1) vari(:,1) ci(:,1:2) skis(:,1) varism(:,1) ci2(:,1:2)];
    xlswrite(filename,A,1,'A1');
    xlswrite(filename,B,1,'A2');
    
    A={'Date','Filtered','var(filtered)','ci_l','ci_u','smoothed','var(smoothed)','ci_l','ci_u'};
    B=[fecha(nk:capt) skif(:,2) vari(:,2) ci(:,3:4) skis(:,2) varism(:,2) ci2(:,3:4)];
    xlswrite(filename,A,2,'A1');
    xlswrite(filename,B,2,'A2');
    
    A={'Date','Filtered','var(filtered)','ci_l','ci_u','smoothed','var(smoothed)','ci_l','ci_u'};
    B=[fecha(nk:capt) skif(:,3) vari(:,3) ci(:,5:6) skis(:,3) varism(:,3) ci2(:,5:6)];
    xlswrite(filename,A,3,'A1');
    xlswrite(filename,B,3,'A2');

    A={'Date','zero','Ex-ante','Ex-post'};
    B=[fecha(nk:capt) zeros(captst,1) ye yv];
    xlswrite(filename,A,4,'A1');
    xlswrite(filename,B,4,'A2');
    
    A={'Startval','Estimates','Std Dev'};
    B=[trans2(startval)',thn,std_thn] ;
    xlswrite(filename,A,5,'A1');
    xlswrite(filename,B,5,'A2');
end
disp('   Initial   Estimates  Std.Dev.');
disp([trans2(startval)',thn,std_thn]);
